# -*- coding: utf-8 -*-
from django.db import models
from django.contrib import messages
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.template.loader import render_to_string
from django.http import HttpResponseRedirect, HttpResponse
from django.core.paginator import Paginator, InvalidPage, EmptyPage
from django.contrib.auth.decorators import login_required
from django.db.models import Q
from django.utils.dates import *

from xml.parsers.expat import ExpatError

from asiou.settings import *
from asiou.educ_proc.models import *
from asiou.educ_proc.forms import *
from asiou.group.models import *
from asiou.properties.models import *
from asiou.properties.forms import *
from asiou.institute.models import *
from asiou.document.models import *
from asiou.common.classes import *
from asiou.common.util import *
from asiou.common.decorators import check_rights
from asiou.common.r_functions import age_calc

import datetime, zipfile, hashlib
from xlwt import *
from collections import *

try:
    import cStringIO as StringIO
except:
    import StringIO

# GTO ranges
GTO_RANGES = {
    6:1,
    7:1,
    8:1,
    9:2,
    10:2,
    11:3,
    12:3,
    13:3,
    14:4,
    15:4,
    16:5,
    17:5,
    18:6,
    19:6,
    20:6,
    21:6,
    22:6,
    23:6,
    24:6,
    25:7,
    26:7,
    27:7,
    28:7,
    29:7,
}

GTO_RANGES_NORM = {
    1:7,
    2:7,
    3:7,
    4:7,
    5:8,
    6:10,
}

# Обработчик для страниц упраления объектами образовательного процесса
@login_required
@check_rights()
def educ_proc(request, object_type, add_filter=None):
    resp_dict = {}
    resp_dict['type'] = object_type
    request_context = RequestContext(request)

    # Проверим есть ли такой тип объектов и получим объекты типа object_type
    type_data = ASIOU_P_ObjectTypes.objects.filter(label=object_type)
    if len(type_data) and type_data[0].class_name in globals():
        type = type_data[0]
        ASIOU_ObjectModel = globals()[type.class_name] # Вернем класс на основе его имени

        # Если это POST, значит запрошено действие
        if request.method == 'POST':
            if request.POST.has_key('delete') and request.POST.has_key('marks'):
                delete_marked(request, ASIOU_ObjectModel, 'delete', 'marks')

                # После POST сделаем редирект на ту же страницу
                return HttpResponseRedirect(request.get_full_path())

        # Запишем модель в сессию и возьмем настроки юзера
        request.session[object_type] = ASIOU_ObjectModel

        # Определимся с количеством записей на страницу
        items_perpage = get_perpage_items(request)

        # Получить список
        filter_params = {
            'type': type.id,
            'institute': request_context['context_institute'],
        }

        # Добавим дополнительные фильтры
        if add_filter != None and len(add_filter):
            filter_params.update(add_filter)

        obj_query_set = ASIOU_ObjectModel.objects.filter(**filter_params)
        table = TableViewer(request, type, ASIOU_ObjectModel, obj_query_set, items_perpage)

        resp_dict['type_data'] = type
        resp_dict['table'] = table
    else:
        resp_dict['unknown_type'] = True

    return render_to_response('educ_proc.html', resp_dict, context_instance=request_context)

# Отображение списка учебных периодов
@login_required
@check_rights()
def ep_shear_range(request, object_type):
    request_context = RequestContext(request)
    add_filter = {'education_year': request_context['edu_year']}
    return educ_proc(request, object_type, add_filter)

# Добавление и изменение учебного периода
@login_required
@check_rights()
def ep_shear_range_add(request, object_type, pk=None, op=None):
    request_context = RequestContext(request)
    add_init = {'education_year': request_context['edu_year']}
    return educ_proc_add(request, object_type, pk, op, add_init)

# Добавление, изменение и копирование записей
@login_required
@check_rights()
def educ_proc_add(request, object_type, pk=None, op=None, add_init=None):
    resp_dict = {}
    form = None
    add_forms = []
    object = None
    resp_dict['errors'] = {}
    resp_dict['type'] = object_type
    resp_dict['pk'] = pk
    resp_dict['op'] = op
    request_context = RequestContext(request)
    context_institute = request_context['context_institute']

    # Проверим есть ли такой тип объектов
    type_data = ASIOU_P_ObjectTypes.objects.filter(label=object_type)
    if len(type_data) and context_institute != None \
        and type_data[0].class_name in globals() \
        and type_data[0].form_class in globals():

        type = type_data[0]
        ASIOU_ObjectModel = globals()[type.class_name] # Вернем класс на основе его имени
        ASIOU_ObjectForm = globals()[type.form_class] # Класс формы
        resp_dict['type_data'] = type

        # Если передан pk, то получим данные
        if pk is not None:
            object_data = ASIOU_ObjectModel.objects.filter(id=pk)
            if len(object_data):
                object = object_data[0]

        # Установить скрытые поля
        post = None
        post_init = {}
        post_init['institute'] = context_institute.id
        post_init['type'] = type.id

        if add_init != None and len(add_init):
            post_init.update(add_init)

        if request.method == 'POST':
            if pk is None or op == 'copy':
                post_init['local_id'] = get_free_id(ASIOU_ObjectModel)
            post = request.POST.copy()
            post.update(post_init)

        # Создать массив дополнительных форм
        add_forms = create_add_forms(type, init_object=object, data=post)

        # Обработка входных данных
        if request.method == 'POST':
            # Если это копирование, то передавать объект не надо
            edit_object = object
            if op is not None and op == 'copy':
                edit_object = None

            # Проверить доп. данные на валидность
            adds_valide = add_forms_valid(add_forms)
            form = ASIOU_ObjectForm(post, instance=edit_object)
            if form.is_valid() and adds_valide:
                saved_object = form.save()
#                saved_object.local_id = saved_object.id
#                saved_object.save()
                for add_form in add_forms:
                    add_form.save(object=saved_object)

                messages.info(request, 'Операция прошла успешно!')
                path_parts = request.path.split('/')
                redirect_url = '/'.join(path_parts[0:3]) + '/edit/' + str(saved_object.id) + '/'
                return HttpResponseRedirect(redirect_url)
            else:
                messages.error(request, 'Форма содержит ошибки!')
        else:
            form = ASIOU_ObjectForm(instance=object)

    elif context_institute == None:
        resp_dict['errors']['unknown_institute'] = True
    else:
        resp_dict['errors']['unknown_type'] = True

    resp_dict['form'] = form
    resp_dict['add_forms'] = add_forms

    return render_to_response('object_add.html', resp_dict, context_instance=request_context)

# Обработчик для страниц упраления учебным планом
@login_required
@check_rights()
def ep_group(request, group_type_id=None):
    resp_dict = {}
    request_context = RequestContext(request)
    context_institute = request_context['context_institute']
    selected_group = None
    resp_dict['has_group'] = False

    object_type = None
    group_type = None

    group_types = ASIOU_Group.objects.filter(institute=context_institute)
    try:
        if group_type_id != None:
            group_type = ASIOU_Group.objects.get(pk=group_type_id, \
                institute=context_institute)
            resp_dict['has_group'] = True
        else:
            group_type = ASIOU_Group.objects.filter(is_main_group=True, \
                institute=context_institute)[0]

        object_type = GROUP_TO_OBJTYPE[group_type.group_type]
    except:
        pass

    resp_dict['type'] = object_type
    resp_dict['group_types'] = group_types
    resp_dict['group_type'] = group_type

    # Проверим есть ли такой тип объектов и получим объекты типа object_type
    type_data = ASIOU_P_ObjectTypes.objects.filter(label=object_type)
    if len(type_data) and type_data[0].class_name in globals():
        type = type_data[0]
        ASIOU_ObjectModel = globals()[type.class_name] # Вернем класс на основе его имени

        # Запишем модель в сессию и возьмем настроки юзера
        request.session[object_type] = ASIOU_ObjectModel
        user_settings = request_context['context_user_settings']

        # Определимся с количеством записей на страницу
        items_perpage = get_perpage_items(request)

        # Получить список
        filter_params = {
            'type': type.id,
            'group_type': group_type,
            'institute': request_context['context_institute'],
            'education_year': request_context['edu_year'],
        }

        # Список групп
        group_list = ASIOU_ObjectModel.objects.filter(**filter_params).order_by('name')
        if request_context['role']:
            if request_context['role'].pk == ROLE_OOU_TEACHER:
                user_id = request_context['context_user_settings'].user.id
                p_id=ASIOU_U_UserProfile.objects.get(user=user_id).person.id
                g_id=ASIOU_EP_Teacher.objects.filter(teach=p_id,group__group__in=group_list.values('id')).values('group__group__id')
                group_list=ASIOU_G_Education.objects.filter(id__in=g_id)

        if len(group_list): selected_group = group_list[0]
        if request.GET.has_key('gid'):
            try:
                selected_group = ASIOU_G_Education.objects.get(pk=request.GET['gid'])
            except:
                pass
        if request_context['role']:
            if request_context['role'].pk == ROLE_OOU_TEACHER:
                ep_g_id = ASIOU_EP_Teacher.objects.filter(teach=p_id,group__group__in=group_list.values('id')).values('group__id')


                group_subjects = ASIOU_EP_Group.objects.filter(
                    id__in=ep_g_id
                    ).order_by('group')
                resp_dict['role']='oou_teacher'

        else:
            # Назначенное преподавание

            group_subjects = ASIOU_EP_Group.objects.filter(
                    institute=request_context['context_institute'],
                    group=selected_group
                ).order_by('order_num')

        resp_dict['group_list'] = group_list
        resp_dict['group_subjects'] = group_subjects

        resp_dict['selected_group'] = selected_group
        resp_dict['type_data'] = type
    else:
        resp_dict['unknown_type'] = True

    # Создать ответ

    return render_to_response('ep_group.html', resp_dict, context_instance=request_context)

# Добавление предмета в учебный план
@login_required
@check_rights()
def ep_group_add(request, object_type, group_type_id=None, pk=None, op=None):
    resp_dict = {}
    form = None
    add_forms = []
    object = None
    resp_dict['errors'] = {}
    resp_dict['pk'] = pk
    resp_dict['op'] = op
    request_context = RequestContext(request)
    context_institute = request_context['context_institute']

    main = False # Класс
    group_object_type = None
    try:
        if group_type_id != None:
            group_type = ASIOU_Group.objects.get(pk=group_type_id, \
                institute=context_institute)
            main = group_type.is_main_group
        else:
            group_type = ASIOU_Group.objects.filter(is_main_group=True, \
                institute=context_institute)[0]
            main = True

        group_object_type = GROUP_TO_OBJTYPE[group_type.group_type]
    except:
        pass

    resp_dict['type'] = object_type
    resp_dict['group_type'] = group_object_type

    # Проверим есть ли такой тип объектов
    type_data = ASIOU_P_ObjectTypes.objects.filter(label=object_type)
    group_type_data = ASIOU_P_ObjectTypes.objects.filter(label=group_object_type)

    if len(type_data) and len(group_type_data) and context_institute != None \
        and type_data[0].class_name in globals() \
        and type_data[0].form_class in globals() \
        and group_type_data[0].class_name in globals():

        type = type_data[0]
        group_type = group_type_data[0]
        ASIOU_ObjectModel = globals()[type.class_name] # Вернем класс на основе его имени
        ASIOU_GroupObjectModel = globals()[group_type.class_name] # Вернем класс группы по имени
        ASIOU_ObjectForm = globals()[type.form_class] # Класс формы
        resp_dict['type_data'] = type

        # Если передан pk, то получим данные
        if pk is not None:
            object_data = ASIOU_ObjectModel.objects.filter(id=pk)
            if len(object_data):
                object = object_data[0]

        # Установить скрытые поля
        post = None
        post_init = {}
        post_init['institute'] = context_institute.id
        post_init['type'] = type.id


        # Установим группу
        if request.GET.has_key('gid'):
            try:
                group = ASIOU_GroupObjectModel.objects.get(pk=request.GET['gid'])
                post_init['group'] = group.pk
            except:
                pass

        if request.method == 'POST':
            if pk is None or op == 'copy':
                post_init['local_id'] = get_free_id(ASIOU_ObjectModel)
            post = request.POST.copy()
            post.update(post_init)

        # Создать массив дополнительных форм

        add_forms = create_add_forms(type, init_object=object, data=post)

        # Обработка входных данных
        if request.method == 'POST':
            # Если это копирование, то передавать объект не надо
            edit_object = object
            if op is not None and op == 'copy':
                edit_object = None

            # Проверить доп. данные на валидность
            adds_valide = add_forms_valid(add_forms)

            form = ASIOU_ObjectForm(post, instance=edit_object)

            if form.is_valid() and adds_valide:

                saved_object = form.save()

#                saved_object.local_id = saved_object.id
#                saved_object.save()
                for add_form in add_forms:
                    add_form.save(object=saved_object)

                messages.info(request, 'Операция прошла успешно!')
                path_parts = request.path.split('/')
                redirect_url = '/'.join(path_parts[0:3]) + '/edit/' + str(saved_object.id) + '/'
                return HttpResponseRedirect(redirect_url)
            else:
                messages.error(request, 'Форма содержит ошибки!')
        else:
            form = ASIOU_ObjectForm(instance=object)

    elif context_institute == None:
        resp_dict['errors']['unknown_institute'] = True
    else:
        resp_dict['errors']['unknown_type'] = True

    resp_dict['form'] = form
    resp_dict['add_forms'] = add_forms

    return render_to_response('object_add.html', resp_dict, context_instance=request_context)

def ep_prog_book_select_otype(request, object_type, ep_group_id):
    resp_dict = {}
    request_context = RequestContext(request)

    type_list1 = ASIOU_P_ObjectTypes.objects.filter(label='dict_programm_add_')
    type_list2=ASIOU_P_ObjectTypes.objects.filter(label='dict_book_add_i')

    # Создать ответ
    resp_dict['type_list1'] = type_list1
    resp_dict['type_list2'] = type_list2

    return render_to_response('select_otype1.html', resp_dict, context_instance=request_context)

# Добавление учителя для преподавания
@login_required
@check_rights()
def ep_group_teacher_add(request, object_type, ep_group_id, pk=None,):
    resp_dict = {}
    form = None
    add_forms = []
    object = None
    resp_dict['errors'] = {}
    resp_dict['pk'] = pk
    request_context = RequestContext(request)
    context_institute = request_context['context_institute']
    resp_dict['type'] = object_type

    # Проверим есть ли такой тип объектов
    type_data = ASIOU_P_ObjectTypes.objects.filter(label=object_type)
    if len(type_data) and context_institute != None \
        and type_data[0].class_name in globals() \
        and type_data[0].form_class in globals():

        type = type_data[0]
        ASIOU_ObjectModel = globals()[type.class_name] # Вернем класс на основе его имени
        ASIOU_ObjectForm = globals()[type.form_class] # Класс формы
        resp_dict['type_data'] = type

        # Если передан pk, то получим данные
        if pk is not None:
            object_data = ASIOU_ObjectModel.objects.filter(id=pk)
            if len(object_data):
                object = object_data[0]

        # Установить скрытые поля
        post = None
        post_init = {}
        post_init['institute'] = context_institute.id
        post_init['type'] = type.id

        # Установим группу по предмету
        group = ASIOU_EP_Group.objects.get(pk=ep_group_id)
        post_init['group'] = group.pk

        if request.method == 'POST':
            if pk is None:
                post_init['local_id'] = get_free_id(ASIOU_ObjectModel)
            post = request.POST.copy()
            post.update(post_init)

        # Создать массив дополнительных форм
        add_forms = create_add_forms(type, init_object=object, data=post)

        # Обработка входных данных
        if request.method == 'POST':
            edit_object = object

            # Проверить доп. данные на валидность
            adds_valide = add_forms_valid(add_forms)
            form = ASIOU_ObjectForm(post, instance=edit_object)
            if form.is_valid() and adds_valide:
                saved_object = form.save()
#                saved_object.local_id = saved_object.id
#                saved_object.save()
                for add_form in add_forms:
                    add_form.save(object=saved_object)

                messages.info(request, 'Операция прошла успешно!')
                path_parts = request.path.split('/')
                redirect_url = '/'.join(path_parts[0:3]) + '/edit/' + str(saved_object.id) + '/'
                return HttpResponseRedirect(redirect_url)
            else:
                messages.error(request, 'Форма содержит ошибки!')
        else:
            form = ASIOU_ObjectForm(instance=object)

    elif context_institute == None:
        resp_dict['errors']['unknown_institute'] = True
    else:
        resp_dict['errors']['unknown_type'] = True

    resp_dict['form'] = form
    resp_dict['add_forms'] = add_forms

    return render_to_response('object_add.html', resp_dict, context_instance=request_context)

# Добавление учебника/программы
@login_required
@check_rights()
def ep_prog_book_add(request, object_type, object_type1, ep_group_id, pk=None,):
    resp_dict = {}
    form = None
    add_forms = []
    object = None
    resp_dict['errors'] = {}
    resp_dict['pk'] = pk
    request_context = RequestContext(request)
    context_institute = request_context['context_institute']
    resp_dict['type'] = object_type
    resp_dict['object_type1'] = object_type1

    # Проверим есть ли такой тип объектов
    type_data = ASIOU_P_ObjectTypes.objects.filter(label=object_type)
    if len(type_data) and context_institute != None \
        and type_data[0].class_name in globals() \
        and type_data[0].form_class in globals():

        type = type_data[0]
        ASIOU_ObjectModel = globals()[type.class_name] # Вернем класс на основе его имени
        ASIOU_ObjectForm = globals()[type.form_class] # Класс формы
        add_param = {}
        if object_type1 == 'dict_book_add_i':
            add_param['add_book_prog'] = ASIOU_I_Books_Programms.objects.filter(type='77')
            add_param['add_desc'] = u'Учебник'
        if object_type1 == 'dict_programm_add_':
            add_param['add_book_prog'] = ASIOU_I_Books_Programms.objects.filter(type='80')
            add_param['add_desc'] = u'Программа'

        resp_dict['type_data'] = type

        # Если передан pk, то получим данные
        if pk is not None:
            object_data = ASIOU_ObjectModel.objects.filter(id=pk)
            if len(object_data):
                object = object_data[0]

        # Установить скрытые поля
        post = None
        post_init = {}
        post_init['institute'] = context_institute.id
        post_init['type'] = type.id

        # Установим группу по предмету
        group = ASIOU_EP_Group.objects.get(pk=ep_group_id)
        post_init['group'] = group.pk

        if request.method == 'POST':
            if pk is None:
                post_init['local_id'] = get_free_id(ASIOU_ObjectModel)

            post_init['prog_book']=request.POST['prog_book']
            post = request.POST.copy()
            post.update(post_init)

        # Создать массив дополнительных форм
        add_forms = create_add_forms(type, init_object=object, data=post)

        # Обработка входных данных
        if request.method == 'POST':
            edit_object = object

            # Проверить доп. данные на валидность
            adds_valide = add_forms_valid(add_forms)
            form = ASIOU_ObjectForm(data=post, instance=edit_object, **add_param)
            if form.is_valid() and adds_valide:
                saved_object = form.save()
#                saved_object.local_id = saved_object.id
#                saved_object.save()
                for add_form in add_forms:
                    add_form.save(object=saved_object)

                messages.info(request, 'Операция прошла успешно!')
                path_parts = request.path.split('/')
                redirect_url = '/'.join(path_parts[0:3])+'/'+ ep_group_id + '/' + object_type1 + '/'+ str(saved_object.id)+'/edit/'
                return HttpResponseRedirect(redirect_url)
            else:
                messages.error(request, 'Форма содержит ошибки!')
        else:
           form = ASIOU_ObjectForm(instance=object, **add_param)
    elif context_institute == None:
        resp_dict['errors']['unknown_institute'] = True
    else:
        resp_dict['errors']['unknown_type'] = True

    resp_dict['form'] = form
    resp_dict['add_forms'] = add_forms

    return render_to_response('object_add.html', resp_dict, context_instance=request_context)

# Обработчик для удаления единичного объекта
@login_required
@check_rights(APP_RIGHT_WRITE)
def educ_proc_delete(request, object_type, pk):
    # Проверим есть ли такой тип объектов и получим объекты типа object_type
    type_data = ASIOU_P_ObjectTypes.objects.filter(label=object_type)
    if len(type_data) and type_data[0].class_name in globals():
        ASIOU_ObjectModel = globals()[type_data[0].class_name]
        try:
            ASIOU_ObjectModel.objects.get(id=pk).delete()
            messages.info(request, 'Операция прошла успешно!')
        except ASIOU_ObjectModel.DoesNotExist:
            messages.error(request, 'Вы пытаетесь удалить объект, которого не существует!')
    else:
        messages.error(request, 'Вы пытаетесь удалить объект неизвестного типа. Обратитесь к администратору!')

    path_parts = request.path.split('/')
    redirect_url = '/'.join(path_parts[0:3]) + '/'
    if request.META.has_key('HTTP_REFERER'):
        redirect_url = request.META['HTTP_REFERER']

    return HttpResponseRedirect(redirect_url)

# Электронный журнал
@login_required
@check_rights()
def educ_proc_marks(request):
    from django.utils.dateformat import format

    request_context = RequestContext(request)
    context_institute = request_context['context_institute']
    resp_dict={}
    show_groups = False
    show_period = False
    show_itog = False
    set_period = False
    groups = None
    start_date = None
    end_date = None
    months = []
    mark_width = 100
    today = datetime.datetime.now()
    tpl = 'marks/marks.html'

    if request.method == 'GET' and request.GET.has_key('p'):
        response = HttpResponseRedirect(request.path)
        response.set_cookie('period', value=request.GET['p'])
        return response

    if request.method == 'GET' and request.GET.has_key('c') and request.GET.has_key('s'):
        response = HttpResponseRedirect(request.path)
        response.set_cookie('class_id', value=request.GET['c'])
        response.set_cookie('subject_id', value=request.GET['s'])
        return response

    if request.COOKIES.has_key('period'):
        period = request.COOKIES['period'].split('_')
        if period[0] == 'p':
            try:
                period = ASIOU_EP_ShearRange.objects.get(pk=period[1], institute=context_institute)
                start_date = period.start_date
                end_date = period.end_date
            except:
                show_period = True

        elif period[0] == 'm':
            p_y, p_m = period[1].split('-')
            p_y = int(p_y)
            p_m = int(p_m)
            start_date = datetime.date(p_y, p_m, 1)
            if p_m == 12:
                p_y += 1
                p_m = 1
            else:
                p_m += 1

            end_date = datetime.date(p_y, p_m, 1) - timedelta(days=1)
            period = {'name':format(start_date, "F Y"),}

        elif period[0] == 'i':
            period = {'name': u'Итоговые'}
            show_itog = True

        else:
            show_period = True

        resp_dict['period'] = period
    else:
        show_period = True

    if request.COOKIES.has_key('class_id') and request.COOKIES.has_key('subject_id'):
        group = None
        subject = None

        # Выбранный класс и предмет
        try:
            group = ASIOU_G_Education.objects.get(id=request.COOKIES['class_id'])
            subject = ASIOU_I_Subject.objects.get(id=request.COOKIES['subject_id'])
            if request_context['role']:
                if request_context['role'].pk == ROLE_OOU_TEACHER:
                    ep_group = []
                    try:
                        groups_t = ASIOU_EP_Teacher.objects.filter(
                            Q(institute=context_institute),
                            Q(teach=request_context['person']),
                            Q(start_date__lte=today),
                            Q(end_date__gte=today) | Q (end_date__isnull=True),
                            Q(group__group=group),
                            Q(group__subject=subject),
                        )

                        class_hour = ASIOU_I_Subject.objects.get(general_code__pk=CLASS_HOUR_ID)
                        ep_group = ASIOU_EP_Group.objects.filter(
                            institute=context_institute,
                            subject=class_hour,
                            start_date__lte=today,
                            group=group,
                            end_date__gte=today,
                            teachers__teach=request_context['person'],
                        )
                    except Exception as e:
                        groups = []

                    ch_groups = []
                    if ep_group:
                        ch_groups = ASIOU_G_Education.objects.filter(
                            Q(institute=context_institute),
                            Q(education_year=request_context['edu_year']),
                            Q(type__label='group_01') | Q(type__label='group_10'),
                            Q(pk__in=ep_group.values('group'))
                        )

                    groups = ASIOU_G_Education.objects.filter(
                        Q(institute=context_institute),
                        Q(education_year=request_context['edu_year']),
                        Q(type__label='group_01') | Q(type__label='group_10'),
                        Q(pk__in=groups_t.values('group__group')) | Q(pk__in=ch_groups)
                    )
        except:
            pass

        if group and subject and (groups == None or group in groups):
            resp_dict['group'] = group
            resp_dict['subject'] = subject
            group_pid = ASIOU_PE_DocumMember.objects.filter(group=group).values('person')
            group_persons = ASIOU_PE_Person.objects.filter(type__label='pup', pk__in=group_pid, via_docum=True).exclude(out=True).order_by('surname', 'name')

            if len(group_persons):
                resp_dict['students'] = group_persons
                ep_group = None
                try:
                    ep_group = ASIOU_EP_Group.objects.get(institute=context_institute, subject=subject, group=group)
                except:
                    messages.error(request, u'Выбранная группа и предмет не найдены!')

                less_filter = {}
                if start_date and end_date:
                    set_period = True
                else:
                    set_period = False
                    start_date = datetime.date(request_context['edu_year'], 9, 1)
                    end_date = datetime.date(request_context['edu_year'] + 1, 9, 1)

                less_filter = {'date__gte':start_date, 'date__lte':end_date}
                resp_dict['ep_group'] = ep_group

                if show_itog:
                    tpl = 'marks/marks_itog.html'
                    resp_dict['all_periods'] = all_periods = ASIOU_EP_ShearRange.objects.filter(education_year=request_context['edu_year'], institute=context_institute)
                    if len(all_periods):
                        mark_width += 35*len(all_periods) + 125
                        marks = {}
                        marks_shear = {}
                        marks_exam = {}
                        pre_marks = ASIOU_EP_ItogMark.objects.filter(institute=context_institute, shear=TYPE_MARK_SREZ, shear_range__in=all_periods, subject=subject, student__in=group_persons, education_year=request_context['edu_year'])
                        itog_marks = ASIOU_EP_ItogMark.objects.filter(institute=context_institute, shear__in=TYPE_MARK_ITOG_DICT, subject=subject, student__in=group_persons, education_year=request_context['edu_year'])
                        exam_period = ASIOU_EP_Exam.objects.filter(institute=context_institute, ep_group=ep_group, **less_filter)
                        if exam_period:
                            mark_width += 35*len(exam_period)

                        exam_marks = ASIOU_EP_ItogMark.objects.filter(institute=context_institute, shear=TYPE_MARK_EXAM, exam__in=exam_period, subject=subject, student__in=group_persons, education_year=request_context['edu_year'])
                        if pre_marks:
                            for m in pre_marks:
                                if not m.shear_range.pk in marks:
                                    marks[m.shear_range.pk] = {}

                                m.name = m.display_name()
                                marks[m.shear_range.pk][m.student.pk] = m

                        if itog_marks:
                            for m in itog_marks:
                                if not m.shear in marks_shear:
                                    marks_shear[m.shear] = {}

                                m.name = m.display_name()
                                marks_shear[m.shear][m.student.pk] = m

                        if exam_marks:
                            for m in exam_marks:
                                if not m.exam.pk in marks_exam:
                                    marks_exam[m.exam.pk] = {}

                                m.name = m.display_name()
                                marks_exam[m.exam.pk][m.student.pk] = m

                        resp_dict['marks'] = marks
                        resp_dict['marks_shear'] = marks_shear
                        resp_dict['marks_exam'] = marks_exam
                        resp_dict['all_exams'] = exam_period
                        resp_dict['mark_width'] = mark_width
                        resp_dict['mark_s_dict'] = TYPE_MARK_ITOG_DICT
                    else:
                        messages.warning(request, u'Не заполнены учебные периоды')

                else:
                    tpl = 'marks/marks.html'
                    lessons = ASIOU_EP_Less.objects.filter(institute=context_institute, ep_group=ep_group, **less_filter).order_by('date')

                    if len(lessons):
                        dates = {}
                        for l in lessons:
                            mark_width += 35
                            dd = int('%d%02d' % (l.date.year, l.date.month))
                            if dd not in dates:
                                months.append((l.date),)
                                dates[dd] = []

                            dates[dd].append(l)

                        desc_type = {}
                        desc_type[0] = [u'Урок', u'У']
                        for item in CHOISE_TYPE_MARKS:
                            desc_type[item[0]] = [item[1], CHOISE_TYPE_MARKS_SHORTS[item[0]]]
                        resp_dict['desc_type'] = desc_type

                        o_less_count = ASIOU_EP_Less_Count_Marks.objects.filter(less__in=lessons)

                        i = 0
                        id_les_count = []
                        less_to_count = {}

                        for item in o_less_count:
                            i += 1
                            if not item.less.pk in less_to_count:
                                less_to_count[item.less.pk] = []
                                i -= 1

                            less_to_count[item.less.pk].append(item)
                            id_les_count.append(item.pk)

                        mark_width += i*35

                        resp_dict['type'] = less_to_count
                        resp_dict['dates'] = sorted(dates.items())

                        marks = {}
                        if set_period:
                            pre_marks = ASIOU_EP_Mark.objects.filter(institute=context_institute, less_count_marks__in=id_les_count)

                            if pre_marks:
                                for m in pre_marks:
                                    if not m.less_count_marks.less.pk in marks:
                                        marks[m.less_count_marks.less.pk] = {}

                                    if not m.less_count_marks.pk in marks[m.less_count_marks.less.pk]:
                                        marks[m.less_count_marks.less.pk][m.less_count_marks.pk] = {}

                                    m.name = m.display_name()
                                    if m.comment is None: m.comment = ''

                                    marks[m.less_count_marks.less.pk][m.less_count_marks.pk][m.student.pk] = m

                        else:
                            resp_dict['students'] = None
                            messages.warning(request, u'Не выбран период для показа оценок')

                        resp_dict['marks'] = marks
                        resp_dict['mark_width'] = mark_width

                        if request.GET.get('print', False):
                            page = 0
                            style = XFStyle()
                            wrappedstyle = easyxf('font: bold off; align: wrap on; alignment: horizontal left;')

                            wb = Workbook()

                            if isinstance(resp_dict['students'], dict) and len(resp_dict['students']):
                                if len(resp_dict['dates']):
                                    temp_flag = 0
                                    count_flag = APP_PRINT_COUNT_MARKS
                                    result = []
                                    temp_dict = {}

                                    for d, months in resp_dict['dates']:
                                        k = 0
                                        temp_dict[d] = []

                                        for l in months:
                                            k += 1
                                            if l.pk in resp_dict['type']:
                                                temp2_flag = len(resp_dict['type'][l.pk])
                                                if count_flag >= (temp_flag + temp2_flag):
                                                    temp_dict[d].append(l)
                                                    temp_flag += temp2_flag

                                                else:
                                                    result.append(sorted(temp_dict.items()))

                                                    temp_flag = len(resp_dict['type'][l.pk])
                                                    temp_dict = {}
                                                    temp_dict[d] = []
                                                    temp_dict[d].append(l)

                                    result.append(sorted(temp_dict.items()))

                                    for r_item in result:
                                        page += 1
                                        ws0 = wb.add_sheet('%s' % page)

                                        # Ячейка с датой
                                        i = 1
                                        j = 0
                                        total_types = 0
                                        ws0.write_merge(i, i + 2, j, j, u'Дата', style)
                                        ws0.col(j).width = 5000


                                        # Генерация ячеек с датой и столбцов в дате
                                        cnt = Counter()
                                        i = 2
                                        j1 = 0
                                        j2 = 0
                                        width_col = 1000

                                        for d, months in r_item:
                                            for l in months:
                                                j1 += 1
                                                j2 += 1
                                                if l.pk in resp_dict['type']:
                                                    j1 -= 1
                                                    l_pk = len(resp_dict['type'][l.pk])
                                                    cnt[d] += l_pk
                                                    total_types += l_pk

                                                    for item in resp_dict['type'][l.pk]:
                                                        j1 += 1
                                                        m = '%s' % resp_dict['desc_type'][item.type][1]
                                                        ws0.write(i + 1, j1, m, style)
                                                        ws0.col(j1).width = width_col

                                                    q = l_pk - 1
                                                    ws0.write_merge(i, i, j2, j2 + q, l.date.day, style)
                                                    j2 += q

                                                else:
                                                    ws0.write(i + 1, j1, '-', style)
                                                    ws0.col(j1).width = width_col
                                                    ws0.write(i, j2, l.date.day, style)

                                        # По какому предмету в классе идет генерация отчета
                                        i = 0
                                        j = 0
                                        ws0.write_merge(i, i, j, total_types, u'%s (%s)' % (resp_dict['ep_group'].group.name, resp_dict['ep_group'].subject.name), style)

                                        # Генерация ячеек с месяцами
                                        i = 1
                                        j = 1
                                        for d, months in r_item:
                                            cnt_value = cnt.get(d)
                                            q = j + cnt_value - 1
                                            d = str(d)
                                            d = '%s.%s' % (d[-2:], d[:4])
                                            ws0.write_merge(i, i, j, q, d, style)
                                            j += q

                                        # Генерация оценок по ученикам
                                        i = 3
                                        for st in resp_dict['students']:
                                            i += 1
                                            j = 0

                                            try:
                                                r = '%s %s.%s.' % (st.surname, st.name[0], st.patronymic[0])
                                            except:
                                                r = '%s %s.' % (st.surname, st.name[0])

                                            ws0.write(i, j, r, style)
                                            for d, days in r_item:
                                                for l in days:
                                                    j += 1
                                                    if l.pk in resp_dict['type']:
                                                        j -= 1
                                                        for item in resp_dict['type'][l.pk]:
                                                            j += 1
                                                            try:
                                                                m = resp_dict['marks'][l.pk][item.pk][st.pk].name
                                                            except:
                                                                m = '-'

                                                            ws0.write(i, j, m, style)

                                                    else:
                                                        ws0.write(i, j, '', style)

                                        i = 50
                                        j = total_types + 4

                                        # Заголовок
                                        ws0.horz_page_breaks = [(i, 0, 255)]
                                        ws0.vert_page_breaks = [(j, 0, 65535)]

                                        ws0.write(i, j, u'Дата', wrappedstyle)

                                        ws0.write(i, j + 1, u'Тема', wrappedstyle)
                                        ws0.col(j + 1).width = 10000

                                        ws0.write(i, j + 2, u'Задание', wrappedstyle)
                                        ws0.col(j + 2).width = 9000

                                        for d, days in r_item:
                                            for l in days:
                                                i += 1
                                                ws0.write(i, j, l.date.strftime('%d.%m.%Y'), wrappedstyle)
                                                ws0.write(i, j + 1, l.theme, wrappedstyle)
                                                ws0.write(i, j + 2, l.home_task, wrappedstyle)

                            # Расшифровка формы выбора
                            page += 1
                            ws0 = wb.add_sheet('%s' % page)
                            ws0.col(0).width = 9000

                            i = -1
                            for k, v in resp_dict['desc_type'].iteritems():
                                i += 1
                                ws0.write(i, 0, u'%s - %s' % (v[1], v[0]), style)

                            today = datetime.datetime.now().strftime('%y%m%d-%H%M')
                            response = HttpResponse()
                            response['Content-Disposition'] = 'attachment; filename=marks(%s).xls' % (today)
                            response['Content-Type'] = 'application/vnd.ms-excel; charset=utf-8'
                            wb.save(response)

                            return response

                    else:
                        if not set_period:
                            resp_dict['students'] = None

                        messages.warning(request, u'В данном классе не проведено уроков')
            else:
                messages.warning(request, u'В данном классе нет учеников')
        else:
            show_groups = True
            messages.warning(request, u'Не верный класс или предмет')
    else:
        show_groups = True

    if show_period:
        periods = []
        for p in ASIOU_EP_ShearRange.objects.filter(education_year=request_context['edu_year'], institute=context_institute):
            periods.append(('p_%s' % p.pk, p.name),)

        p_choices = [('-', ''),(u'Учебные периоды', periods),]
        if months:
            p_months = []
            for m in months:
                p_months.append(('m_%d-%02d' % (m.year, m.month), format(m, "F Y")))

            p_choices.append((u'Месяцы', p_months))

        p_choices.append(('i_0', u'Итоговые'))
        resp_dict['periods'] = Select().render('p', None, attrs={'onchange':'selectPeriod(this);'}, choices=p_choices)

    if show_groups:
        if request_context['role']:
            if request_context['role'].pk == ROLE_OOU_TEACHER:
                ep_group = []
                try:
                    groups_t = ASIOU_EP_Teacher.objects.filter(
                        Q(institute=context_institute),
                        Q(teach=request_context['person']),
                        Q(start_date__lte=today),
                        Q(end_date__gte=today) | Q (end_date__isnull=True)
                    )

                    class_hour = ASIOU_I_Subject.objects.get(general_code__pk=CLASS_HOUR_ID)
                    ep_group = ASIOU_EP_Group.objects.filter(
                        institute=context_institute,
                        subject=class_hour,
                        start_date__lte=today,
                        end_date__gte=today,
                        teachers__teach=request_context['person'],
                    )
                except Exception as e:
                    messages.warning(request, u'В локальном справочнике нет классного часа')
                    groups = []

                ch_groups = []
                if ep_group:
                    ch_groups = ASIOU_G_Education.objects.filter(
                        Q(institute=context_institute),
                        Q(education_year=request_context['edu_year']),
                        Q(type__label='group_01') | Q(type__label='group_10'),
                        Q(pk__in=ep_group.values('group'))
                    )

                groups = ASIOU_G_Education.objects.filter(
                    Q(institute=context_institute),
                    Q(education_year=request_context['edu_year']),
                    Q(type__label='group_01') | Q(type__label='group_10'),
                    Q(pk__in=groups_t.values('group__group')) | Q(pk__in=ch_groups)
                )
            else:
                groups = ASIOU_G_Education.objects.filter(
                    Q(institute=context_institute),
                    Q(education_year=request_context['edu_year']),
                    Q(type__label='group_01') | Q(type__label='group_10')
                )
        else:
            groups = ASIOU_G_Education.objects.filter(
                Q(institute=context_institute),
                Q(education_year=request_context['edu_year']),
                Q(type__label='group_01') | Q(type__label='group_10')
            )

        if not groups:
            messages.warning(request, u'У вас нет классов в которых вы преподаете')

        resp_dict['groups'] = groups

    return render_to_response(tpl, resp_dict, context_instance=request_context)

# Комплекс ГТО
@login_required
@check_rights()
def educ_proc_gto(request):
    request_context = RequestContext(request)
    context_institute = request_context['context_institute']
    resp_dict={}
    show_groups = False
    groups = None
    today = datetime.datetime.now()

    if request.COOKIES.has_key('gto_group_id'):
        group = None

        # Выбранный класс
        try:
            group = ASIOU_G_Education.objects.get(id=request.COOKIES['gto_group_id'])
            if request_context['role']:
                if request_context['role'].pk == ROLE_OOU_TEACHER:
                    try:
                        gto_subj = ASIOU_I_Subject.objects.get(general_code__pk=GTO_ID)
                        ep_group = ASIOU_EP_Group.objects.filter(
                            institute=context_institute,
                            subject=gto_subj,
                            start_date__lte=today,
                            end_date__gte=today,
                            teachers__teach=request_context['person'],
                        )

                        groups = ASIOU_G_Education.objects.filter(
                            Q(institute=context_institute),
                            Q(education_year=request_context['edu_year']),
                            Q(type__label='group_01') | Q(type__label='group_10'),
                            Q(pk__in=ep_group.values('group'))
                        )
                    except Exception as e:
                        print e
                        groups = []

                    if not group in groups:
                        group = None;
        except:
            pass

        if group:
            resp_dict['group'] = group
            group_pid = ASIOU_PE_DocumMember.objects.filter(group=group).values('person')
            group_persons = ASIOU_PE_Person.objects.filter(type__label='pup', pk__in=group_pid, via_docum=True).exclude(out=True).order_by('surname', 'name')

            students = {}
            prizes = {}
            if len(group_persons):
                for p in group_persons:
                    #start_date = datetime.date(request_context['edu_year'], 9, 1)
                    #p.age =  request_context['edu_year'] - p.birth_date.year
                    #if (datetime.date(request_context['edu_year'], p.birth_date.month, p.birth_date.day)-start_date).days > 0:
                    #    p.age = p.age-1
                    p.age = age_calc(p.birth_date.year,p.birth_date.month,p.birth_date.day,request_context['edu_year'], 9, 1,request)
                    if p.age in GTO_RANGES:
                        if GTO_RANGES[p.age] not in students:
                            students[GTO_RANGES[p.age]] = []

                        students[GTO_RANGES[p.age]].append(p)

                resp_dict['students'] = students

                gtos = {}
                gto_width = {}
                all_gto = ASIOU_Dict_GTO.objects.filter(level__in=students.keys())
                if len(all_gto):
                    gto_vals = {}
                    for gto in all_gto:
                        if gto.ch_values:
                            g_vals = []
                            for g_v in gto.ch_values.split(';'):
                                v = g_v.split(':')
                                g_vals.append((int(v[0]),v[1]),)

                                if not gto.pk in gto_vals:
                                    gto_vals[gto.pk] = {}

                                gto_vals[int(gto.pk)][int(v[0])] = v[1]

                            gto.ch_values = g_vals

                        if gto.level not in gtos:
                            gtos[gto.level] = []

                        if not gto.level in gto_width:
                            gto_width[gto.level] = 104+47+47+47+5

                        gto_width[gto.level] += 47
                        gtos[gto.level].append(gto)

                    resp_dict['g_width'] = gto_width
                    resp_dict['gtos'] = gtos

                    marks={}
                    pre_marks = ASIOU_PE_GTO_Result.objects.filter(institute=context_institute, gto__in=all_gto, edu_year=request_context['edu_year'], person__in=group_pid)
                    if pre_marks:
                        for m in pre_marks:
                            if not m.gto.pk in marks:
                                marks[m.gto.pk] = {}

                            if m.person.pk not in prizes:
                                prizes[m.person.pk] = {1:0, 2:0, 0:GTO_RANGES_NORM[m.gto.level]}

                            if m.person.sex == PERSON_SEX_MALE:
                                s = 'boy'
                            elif m.person.sex == PERSON_SEX_FEMALE:
                                s = 'girl'

                            gold = getattr(m.gto, 'gold_'+s)
                            silver = getattr(m.gto, 'silver_'+s)
                            if gold and silver:
                                if _have_medal(m.value, gold, m.gto.sign):
                                    m.prize = 1
                                elif _have_medal(m.value, silver, m.gto.sign):
                                    m.prize = 2
                                else:
                                    m.prize = 0

                                if m.prize:
                                    if not 'all' in prizes:
                                        prizes['all'] = {1:0, 2:0,'level':{}}

                                    if not m.gto.level in prizes['all']['level']:
                                        prizes['all']['level'][m.gto.level] = {1:0, 2:0,}

                                    if m.person in students[m.gto.level]:
                                        prizes[m.person.pk][m.prize]+=1
                                        prizes['all'][m.prize]+=1
                                        prizes['all']['level'][m.gto.level][m.prize]+=1
                                        prizes[m.person.pk][0]-=1
                            else:
                                m.prize = 0

                            try:
                                m.t_value = gto_vals[m.gto.pk][int(m.value)]
                                m.value = int(m.value)
                            except Exception as e:
                                m.t_value = None

                            marks[m.gto.pk][m.person.pk] = m

                    resp_dict['prizes'] = prizes
                    resp_dict['marks'] = marks
                else:
                    messages.warning(request, u'Справочник норм ГТО для данного класса пуст!')
            else:
                messages.warning(request, u'В данном классе нет учеников')
        else:
            show_groups = True
            messages.warning(request, u'Не верный класс')
    else:
        show_groups = True

    if show_groups:
        if request_context['role']:
            if request_context['role'].pk == ROLE_OOU_TEACHER:
                try:
                    gto_subj = ASIOU_I_Subject.objects.get(general_code__pk=GTO_ID)
                    ep_group = ASIOU_EP_Group.objects.filter(
                        institute=context_institute,
                        subject=gto_subj,
                        start_date__lte=today,
                        end_date__gte=today,
                        teachers__teach=request_context['person'],
                    )

                    groups = ASIOU_G_Education.objects.filter(
                        Q(institute=context_institute),
                        Q(education_year=request_context['edu_year']),
                        Q(type__label='group_01') | Q(type__label='group_10'),
                        Q(pk__in=ep_group.values('group'))
                    )
                except Exception as e:
                    print e
                    groups = []
        else:
            groups = ASIOU_G_Education.objects.filter(
                institute=context_institute,
                education_year=request_context['edu_year'],
                type__label='group_01'
            )

        if not groups:
            messages.warning(request, u'У вас нет классов в которых вы ведете ГТО')

        resp_dict['groups'] = groups

    return render_to_response('marks/gto.html', resp_dict, context_instance=request_context)

def _have_medal(value, norm, sign):
    if sign:
        if value >= norm:
            return True
    else:
        if value <= norm:
            return True

    return False

# Награжденные в комплексе ГТО
@login_required
@check_rights()
def educ_proc_gto_list(request):
    request_context = RequestContext(request)
    context_institute = request_context['context_institute']
    resp_dict={}

    all_marks = ASIOU_PE_GTO_Result.objects.filter(institute=context_institute, edu_year=request_context['edu_year'])
    prizes = {}
    prizes_all = {}
    if all_marks:
        for m in all_marks:
            # Проверка на соответствие возраста и ступени
            m.age = age_calc(m.person.birth_date.year,m.person.birth_date.month,m.person.birth_date.day,request_context['edu_year'], 9, 1,request)
            if GTO_RANGES[m.age] == m.gto.level:
                if m.person.pk not in prizes:
                    prizes[m.person.pk] = {1:0, 2:0, 0:GTO_RANGES_NORM[m.gto.level], 'level':m.gto.level}

                if m.person.sex == PERSON_SEX_MALE:
                    s = 'boy'
                elif m.person.sex == PERSON_SEX_FEMALE:
                    s = 'girl'

                gold = getattr(m.gto, 'gold_'+s)
                silver = getattr(m.gto, 'silver_'+s)
                if gold and silver:
                    if _have_medal(m.value, gold, m.gto.sign):
                        m.prize = 1
                    elif _have_medal(m.value, silver, m.gto.sign):
                        m.prize = 2
                    else:
                        m.prize = 0

                    if m.prize:
                        if not prizes_all:
                            prizes_all = {1:0, 2:0,'level':{}}

                        prizes[m.person.pk][m.prize]+=1
                        prizes_all[m.prize]+=1
                        prizes[m.person.pk][0]-=1
                else:
                    m.prize = 0

        gold = {}
        silver = {}
        group_persons = ASIOU_PE_Person.objects.filter(pk__in=prizes.keys(), via_docum=True).exclude(out=True).order_by('surname', 'name')
        persones = reindex_by(group_persons, 'id', True)

        pre_groups = ASIOU_G_Education.objects.filter(
            institute=context_institute,
            education_year=request_context['edu_year'],
            type__label='group_01'
        )
        documents = ASIOU_PE_DocumMember.objects.filter(person__in=group_persons, group__in=pre_groups)

        group_pid = {}
        for doc in documents:
            try:
                group_pid[doc.person.pk] = doc.group.pk
            except:
                pass

        pre_groups = pre_groups.filter(id__in=group_pid.values())
        groups = reindex_by(pre_groups, 'id', True)
        medal_all = {1:0, 2:0}
        for person in prizes:
            if person in persones:
                if prizes[person][0] <= 0:
                    persones[person].level = prizes[person]['level']
                    if prizes[person][2] < 3:
                        if not group_pid[person] in gold:
                            gold[group_pid[person]] = []

                        gold[group_pid[person]].append(persones[person])
                        medal_all[1] += 1
                    else:
                        if not group_pid[person] in silver:
                            silver[group_pid[person]] = []

                        silver[group_pid[person]].append(persones[person])
                        medal_all[2] += 1

        resp_dict['medal_all'] = medal_all
        resp_dict['prizes_all'] = prizes_all
        resp_dict['gold'] = gold
        resp_dict['silver'] = silver
        resp_dict['groups'] = groups

    return render_to_response('marks/gto_list.html', resp_dict, context_instance=request_context)

# Ошибки возраста в комплексе ГТО
@login_required
@check_rights()
def educ_proc_gto_list_err(request):
    request_context = RequestContext(request)
    context_institute = request_context['context_institute']
    resp_dict={}
    pers_err = {}
    pers_all  = {}

    all_marks = ASIOU_PE_GTO_Result.objects.filter(institute=context_institute, edu_year=request_context['edu_year'])
    if all_marks:
        for m in all_marks:
            if m.person.pk not in pers_all:
               m.age = age_calc(m.person.birth_date.year,m.person.birth_date.month,m.person.birth_date.day,request_context['edu_year'], 9, 1,request)
               if GTO_RANGES[m.age] == m.gto.level:
                   pers_all[m.person.pk] = {1:0, 2:0, 0:GTO_RANGES_NORM[m.gto.level], 'level':m.gto.level}
               else:
                   m.gto.level = GTO_RANGES[m.age]
                   pers_all[m.person.pk] = {1:0, 2:0, 0:GTO_RANGES_NORM[m.gto.level], 'level':m.gto.level}
                   pers_err[m.person.pk] = {1:0, 2:0, 0:GTO_RANGES_NORM[m.gto.level], 'level':m.gto.level}

        gto_err = {}
        group_persons = ASIOU_PE_Person.objects.filter(pk__in=pers_err.keys(), via_docum=True).exclude(out=True).order_by('surname', 'name')
        persones = reindex_by(group_persons, 'id', True)

        pre_groups = ASIOU_G_Education.objects.filter(
            institute=context_institute,
            education_year=request_context['edu_year'],
            type__label='group_01'
        )
        documents = ASIOU_PE_DocumMember.objects.filter(person__in=group_persons, group__in=pre_groups)

        group_pid = {}
        for doc in documents:
            try:
                group_pid[doc.person.pk] = doc.group.pk
            except:
                pass

        pre_groups = pre_groups.filter(id__in=group_pid.values())
        groups = reindex_by(pre_groups, 'id', True)

        err_all = 0
        for person in pers_err:
            if person in persones:
               if not group_pid[person] in gto_err:
                  gto_err[group_pid[person]] = []

               gto_err[group_pid[person]].append(persones[person])
               err_all += 1

        resp_dict['err_all'] = err_all
        resp_dict['gto_err'] = gto_err
        resp_dict['groups'] = groups

    return render_to_response('marks/gto_list_err.html', resp_dict, context_instance=request_context)

#Импортирование в дневник
@login_required
@check_rights()
def rid_export(request):
    from time import time
    start = time()

    request_context = RequestContext(request)
    context_institute = request_context['context_institute']
    type_data = ASIOU_P_ObjectTypes.objects.filter(label='work')
    today = datetime.datetime.now()

    path_parts = request.path.split('/')
    redirect_url = '/'
    if request.META.has_key('HTTP_REFERER'):
        redirect_url = request.META['HTTP_REFERER']

    resp_dict = {}
    resp_dict['teach'] = {}
    resp_dict['marks'] = {}
    resp_dict['today'] = today

    start_date = edu_date_start = datetime.date(request_context['edu_year'], settings.APP_EDU_YEAR_BORDER_MONTH, settings.APP_EDU_YEAR_BORDER_DAY)
    end_date = edu_date_end = datetime.date(request_context['edu_year']+1, settings.APP_EDU_YEAR_BORDER_MONTH, settings.APP_EDU_YEAR_BORDER_DAY)
    if request.method == 'POST':
        post = request.POST.copy()
        form = ASIOU_EP_RID_Export(post)
        if form.is_valid():
            if form.cleaned_data['date']:
                if form.cleaned_data['date']['start']:
                    start_date = form.cleaned_data['date']['start']
                    if start_date < edu_date_start:
                        messages.error(request, u'Не корректная дата начала, она должна быть позже даты начала учебного года!')
                        return HttpResponseRedirect(redirect_url)

                    if start_date >= edu_date_end:
                        messages.error(request, u'Не корректная дата начала, она должна быть раньше даты окончания учебного года!')
                        return HttpResponseRedirect(redirect_url)

                if form.cleaned_data['date']['end']:
                    end_date = form.cleaned_data['date']['end']
                    if end_date >= edu_date_end:
                        messages.error(request, u'Не корректная дата окончания, она должна быть раньше даты окончания учебного года!')
                        return HttpResponseRedirect(redirect_url)

                    if end_date < edu_date_start:
                        messages.error(request, u'Не корректная дата окончания, она должна быть позже даты начала учебного года!')
                        return HttpResponseRedirect(redirect_url)

                try:
                    class_hour = ASIOU_I_Subject.objects.get(general_code__pk=CLASS_HOUR_ID)
                except:
                    messages.warning(request, u'Не занесен классный час в справочник!')
                    return HttpResponseRedirect(redirect_url)

                ep_groups = ASIOU_EP_Group.objects.filter(
                    institute=context_institute,
                    start_date__lte=end_date,
                    end_date__gte=start_date,
                    group__education_year=request_context['edu_year'],
                )
                teachers = ASIOU_EP_Teacher.objects.filter(
                    institute=context_institute,
                    group__in=ep_groups,
                    start_date__lte=end_date,
                    end_date__gte=start_date,
                ).order_by('-replace').values()
                teach_list = list_of(teachers, 'teach_id')
                t_person = ASIOU_PE_Person.objects.filter(
                    institute=context_institute,
                    pk__in = teach_list
                ).values()
                t_person = reindex_by(t_person, 'id')
                teach_file_data_teachers = {}

                for t in teachers:
                    if not t['teach_id'] in teach_file_data_teachers:
                        if t['teach_id'] in t_person:
                            teach_file_data_teachers[t['teach_id']] = {
                                'id':t['teach_id'],
                                'login':t_person[t['teach_id']]['login'],
                                'pass':hashlib.md5(t_person[t['teach_id']]['password']).hexdigest(),
                                'value':'%s %s %s' % (
                                    t_person[t['teach_id']]['surname'],
                                    t_person[t['teach_id']]['name'],
                                    t_person[t['teach_id']]['patronymic']
                                )
                            }

                resp_dict['teach']['teachers'] = teach_file_data_teachers
                del teach_file_data_teachers, t
                teachers = reindex_by(teachers, 'group_id', False, True)
                # А снег все падал и падал...
                groups = {}
                subject_ids = []

                for ep_g in ep_groups:
                    keys = []
                    if ep_g.group.group_type.is_main_group:
                        keys = [ep_g.group]
                    else:
                        for g_ed in ep_g.group.from_groups.select_related():
                            keys.append(g_ed)

                    for key in keys:
                        if not key in groups:
                            groups[key] = []

                        groups[key].append(ep_g)

                    if not ep_g.subject.pk in subject_ids:
                        subject_ids.append(ep_g.subject.pk)
                all_groups = ASIOU_G_Education.objects.filter(
                    Q(institute=context_institute),
                    Q(education_year=request_context['edu_year']),
                    Q(type__label='group_01') | Q(type__label='group_10')
                )
                # Предметы
                subjects = ASIOU_I_Subject.objects.filter(pk__in=subject_ids)
                teach_file_data_subjects = []
                for s in subjects:
                    teach_file_data_subjects.append({'id':s.pk, 'value':s.name})

                resp_dict['teach']['subjects'] = teach_file_data_subjects
                del teach_file_data_subjects, subjects
                # Образовательные периоды
                all_periods = ASIOU_EP_ShearRange.objects.filter(education_year=request_context['edu_year'], institute=context_institute).values()
                all_periods = reindex_by(all_periods, 'id')
                teach_file_data_shr_ranges = []
                for shr in all_periods.values():
                    teach_file_data_shr_ranges.append({'id':shr['id'], 'start_date':shr['start_date'], 'end_date':shr['end_date'], 'value':shr['name']})

                resp_dict['teach']['shr_ranges'] = teach_file_data_shr_ranges
                del teach_file_data_shr_ranges
                documents = ASIOU_PE_DocumMember.objects.filter(group__in=all_groups).values()
                documents_person = list_of(documents, 'person_id')
                group_docs = reindex_by(documents, 'group_id', False, True)
                group_persons = ASIOU_PE_Person.objects.filter(institute=context_institute, type__label='pup', pk__in=documents_person, via_docum=True, not_export=False).distinct().values()
                group_persons_pk = list_of(group_persons, 'id')
                group_persons = reindex_by(group_persons, 'id')
                lessons = ASIOU_EP_Less.objects.filter(
                    institute=context_institute,
                    ep_group__in=ep_groups,
                    date__gte=start_date,
                    date__lte=end_date
                ).values()
                lessons_pk = list_of(lessons, 'id')
                less_count_marks = ASIOU_EP_Less_Count_Marks.objects.filter(
                    institute=context_institute,
                    less__in=lessons_pk,
                ).values()
                less_count_marks_id = list_of(less_count_marks, 'id')
                pre_marks = ASIOU_EP_Mark.objects.filter(
                    institute=context_institute,
                    less_count_marks__in=less_count_marks_id,
                    student__in=group_persons_pk
                ).values()
                lessons = reindex_by(lessons, 'ep_group_id', False, True)
                marks = reindex_by(pre_marks, 'less_count_marks_id', False, True)
                less_count_marks = reindex_by(less_count_marks, 'less_id')
                del pre_marks, all_groups, ep_groups
                teach_file_data_teach = {}
                teach_file_data_edgrp = []
                student_file_data_student = {}
                student_file_data_student_reverse = {}
                schedule_file_data = {}
                marks_file_data = {}
                stud_in_group = {}

                for group, ep_gs in groups.items():
                    epgrps_in_group = []
                    teach_file_data_teach[group] = []
                    student_file_data_student[group] = {}
                    schedule_file_data[group] = {}
                    for ep_g in ep_gs:
                        if not ep_g.group_id in epgrps_in_group:
                            epgrps_in_group.append(ep_g.group_id)

                        if not ep_g.pk in teachers:
                            messages.warning(request, u'Для учебной группы %s  не назначен учитель!' % ep_g)
                            #return HttpResponseRedirect(redirect_url)
                        else:
                            edgrp = None
                            if not ep_g.group == group:
                                #Узнать как выглядит год и добавить его в нормальном виде
                                edgrp_data = {'id':ep_g.group.pk, 'year':ep_g.group.education_year, 'name':ep_g.group.name}
                                teach_file_data_edgrp.append(edgrp_data)
                                edgrp = ep_g.group.pk

                            if group.pk in group_docs:
                                for doc in group_docs[group.pk]:
                                    if doc['person_id'] in group_persons:
                                        if group_persons[doc['person_id']]['id'] in student_file_data_student[group]:
                                            if edgrp and not edgrp in student_file_data_student[group][group_persons[doc['person_id']]['id']]['edgrp']:
                                                student_file_data_student[group][group_persons[doc['person_id']]['id']]['edgrp'].append(edgrp)
                                                student_file_data_student_reverse[group_persons[doc['person_id']]['id']] = group.pk
                                        else:
                                            st = {
                                                'id':group_persons[doc['person_id']]['id'],
                                                'login':group_persons[doc['person_id']]['login'],
                                                'pass':hashlib.md5(group_persons[doc['person_id']]['password']).hexdigest(),
                                                'edgrp':[]
                                            }
                                            if edgrp:
                                                st['edgrp'] = [edgrp]

                                            student_file_data_student[group][group_persons[doc['person_id']]['id']] = st

                                        if group in stud_in_group:
                                            if doc['person_id'] not in stud_in_group[group]:
                                                stud_in_group[group].append(doc['person_id'])
                                        else:
                                            stud_in_group[group] = [doc['person_id']]

                                if ep_g.pk in lessons:
                                    for l in lessons[ep_g.pk]:
                                        if not l['date'] in marks_file_data:
                                            marks_file_data[l['date']] = {}

                                        if not group in marks_file_data[l['date']]:
                                            marks_file_data[l['date']][group] = {}
                                            marks_file_data[l['date']][group]['marks'] = {}
                                            marks_file_data[l['date']][group]['less'] = []
                                            marks_file_data[l['date']][group]['work'] = []

                                        if l['id'] in less_count_marks and less_count_marks[l['id']]['id'] in marks:
                                            less_marks = reindex_by(marks[less_count_marks[l['id']]['id']], 'student_id', False, True)
                                            for st, ms in less_marks.items():

                                                if group in stud_in_group and st in stud_in_group[group]:
                                                    if st not in marks_file_data[l['date']][group]['marks']:
                                                        marks_file_data[l['date']][group]['marks'][st] = []

                                                    for m in ms:
                                                        marks_file_data[l['date']][group]['marks'][st].append({
                                                            'lesson':l['id'],
                                                            'id':m['id'],
                                                            'value':m['mark']
                                                        })

                                        less_info = {'id': l['id'], 'num': l['order_num'], 'subject': ep_g.subject.pk, 'theme': l['theme']}
                                        work_info = {'lesson':l['id'], 'ondate':l['date'], 'value':l['home_task']}
                                        if edgrp:
                                            less_info['edgrp'] = edgrp
                                            work_info['edgrp'] = edgrp

                                        marks_file_data[l['date']][group]['less'].append(less_info)
                                        marks_file_data[l['date']][group]['work'].append(work_info)

                            for t in teachers[ep_g.pk]:
                                v = {'id':t['teach_id'], 'start_date':t['start_date'], 'end_date':t['end_date']}
                                if class_hour == ep_g.subject:
                                    v['classmanager'] = 1
                                else:
                                    v['subject'] = ep_g.subject.pk

                                if edgrp:
                                    v['edgrp'] = edgrp

                                teach_file_data_teach[group].append(v)

                        # Расписание для всех групп в классе
                        schedule = ASIOU_EP_Timetable.objects.filter(institute=context_institute, ep_group__group=ep_g.group_id)#__in=epgrps_in_group
                        for doc in schedule:
                            edgrp = None
                            day_week = doc.day_week

                            if not day_week in schedule_file_data[group]:
                                schedule_file_data[group][day_week] = {}
                            st = {'ep_group':doc.ep_group.subject.pk, 'order_num':doc.order_num, 'room':'не указан', 'edgrp':0}

                            if doc.room:
                                st['room'] = doc.room

                            if doc.ep_group.group.type_id == 28:
                                st['edgrp'] = doc.ep_group.group.pk

                            schedule_file_data[group][day_week][doc.id] = st

                del groups, lessons, teachers, group_docs, marks, documents, stud_in_group, schedule, epgrps_in_group


                resp_dict['teach']['teach'] = teach_file_data_teach
                resp_dict['teach']['edgrp'] = teach_file_data_edgrp
                del teach_file_data_teach, teach_file_data_edgrp

                resp_dict['student'] = student_file_data_student
                resp_dict['schedule'] = schedule_file_data
                resp_dict['marks'] = marks_file_data
                del student_file_data_student, marks_file_data, schedule_file_data

                itog_marks = ASIOU_EP_ItogMark.objects.filter(institute=context_institute, education_year=request_context['edu_year'], student__in=group_persons).values()
                del group_persons

                # Итоговые отметки
                itog_marks_file_data = {}

                for i_marks in itog_marks:

                    pk_shr_range = i_marks['shear_range_id'] if i_marks['shear_range_id'] else None
                    pk_exam = i_marks['exam_id'] if i_marks['exam_id'] else None

                    pk_mark = i_marks['id']
                    pk_st = i_marks['student_id']
                    group = student_file_data_student_reverse.get(pk_st, False)
                    subject = i_marks['subject_id']

                    if group:
                        if subject in subject_ids:
                            if not group in itog_marks_file_data:
                                itog_marks_file_data[group] = {}
                                itog_marks_file_data[group]['students'] = {}
                                itog_marks_file_data[group]['exams'] = {}

                            if not pk_st in itog_marks_file_data[group]['students']:
                                itog_marks_file_data[group]['students'][pk_st] = {}

                            itog_marks_file_data[group]['students'][pk_st][pk_mark] = {
                                'id':pk_mark,
                                'subject':i_marks['subject_id'],
                                'mark':i_marks['mark'],
                                'type':i_marks['shear'],
                                'shear_range':pk_shr_range,
                                'exam':pk_exam
                            }

                            if pk_exam:
                                if not pk_exam in itog_marks_file_data[group]['exams']:
                                    edgrp_exam = i_marks.exam.ep_group.group.pk
                                    temp_exam = {'ondate':i_marks.exam.date, 'value':i_marks.exam.name}

                                    if not group == edgrp_exam:
                                       temp_exam['edgrp'] = edgrp_exam

                                    itog_marks_file_data[group]['exams'][pk_exam] = temp_exam

                resp_dict['itog_marks'] = itog_marks_file_data
                del itog_marks_file_data, itog_marks, student_file_data_student_reverse, subject_ids

                expmark = {}
                expmark['expmark-Teach.xml'] = render_to_string('rid_import/expmark-Teach.xml', resp_dict, context_instance=request_context)
                expmark['expmark-Student.xml'] = render_to_string('rid_import/expmark-Student.xml', resp_dict, context_instance=request_context)
                expmark['expmark-Current.xml'] = render_to_string('rid_import/expmark-Current.xml', resp_dict, context_instance=request_context)
                expmark['expmark-Schedule.xml'] = render_to_string('rid_import/expmark-Schedule.xml', resp_dict, context_instance=request_context)
                expmark['expmark-Itog.xml'] = render_to_string('rid_import/expmark-Itog.xml', resp_dict, context_instance=request_context)

                o_xmls = {}
                import xml.etree.ElementTree as ElementTree
                for xml_file, xml_str in expmark.items():
                    try:
                        o_xml = ElementTree.ElementTree(ElementTree.fromstring(xml_str.encode('utf-8')))
                        o_xmls[xml_file] = o_xml
                        o_xml.write(os.path.join( APP_RID_EXPORT_DIR, xml_file))

                    except Exception as e:
                        messages.error(request, u'Ошибка в формате XML-файла "%s" : %s' % (xml_file, e))
                        return HttpResponseRedirect(redirect_url)

                del expmark, o_xml
                print time()-start
                # Открываем StringIO, чтобы захватить в памяти содержимое ZIP
                s = StringIO.StringIO()

                zipfile_name = 'expmark-%s.zip' % today.strftime('%y%m%d-%H%M')

                # Объект архива
                archive = zipfile.ZipFile(s, 'w')
                try:
                    for xml_file in o_xmls:
                        # Путь до файла во временной папке
                        fpath = os.path.join(APP_RID_EXPORT_DIR, xml_file)

                        # Добавляем файл по заданному пути
                        archive.write(fpath, xml_file)
                    archive.close()
                except Exception as e:
                    messages.error(request, u'Не удается создать архив, error: %s ' % e)
                    return HttpResponseRedirect(redirect_url)

                response = HttpResponse(s.getvalue(), mimetype = "application/x-zip-compressed")
                response['Content-Disposition'] = 'attachment; filename=%s' % zipfile_name
                response['Content-Type'] = 'application/zip; charset=utf-8'
                print time()-start
                return response
            else:
                messages.error(request, u'Необходимо выбрать хотябы 1 дату!')

        else:
            messages.error(request, 'Форма содержит ошибки!')
    else:
        form = ASIOU_EP_RID_Export()

    resp_dict['form'] = form
    return render_to_response('marks/marks_export.html', resp_dict, context_instance=request_context)

# Управление связанными объектами
@login_required
@check_rights()
def educ_proc_related(request, object_type, pk, rel_model, rel_pk=None):
    return object_related(request, object_type, pk, rel_model, rel_pk)

@login_required
@check_rights(APP_RIGHT_WRITE)
def educ_proc_related_delete(request, object_type, pk, rel_model, rel_pk):
    return object_related_delete(request, object_type, pk, rel_model, rel_pk)

#Экспорт ГТО
@login_required
@check_rights()
def GTO_export(request):
    request_context = RequestContext(request)
    context_institute = request_context['context_institute']
    year= request_context['edu_year']

    try:
        file=open(os.path.join( APP_RID_EXPORT_DIR, 'GTO_Result_'+str(context_institute.id)+'.txt'), 'w')
    except IOError:
        print('Can\'t open the file {0}')
    else:
        gto_result=ASIOU_PE_GTO_Result.objects.filter()
        for g in gto_result:
            institute=g.institute.id
            group_pid=ASIOU_PE_DocumMember.objects.filter((Q(docum__type__label='doc_pup_in')|Q(docum__type__label='doc_pup_mov')|Q(docum__type__label='doc_pup_mov_bed2')),Q(person=g.person)).order_by('docum__docum_date')
            group_id=group_pid[group_pid.count()-1].group.id
            class_id=group_id
            class_name=ASIOU_G_Education.objects.get(institute=context_institute,id=group_id).name
            level=ASIOU_Dict_GTO.objects.get(id=g.gto.id).level
            person=g.person.id
            sex=ASIOU_PE_Person.objects.get(id=person,institute=context_institute).sex
            bithdate=ASIOU_PE_Person.objects.get(id=person,institute=context_institute).birth_date
            gto_id=g.gto.id
            gto_name=g.gto
            value=g.value
            file.write(str(institute)+';'+str(class_id)+';'+str(class_name)+';'+str(level)+';'+str(person)+';'+str(sex)+';'+str(bithdate)+';'+str(gto_id)+';'+str(force_unicode(gto_name))+';'+str(value)+';\n')
    finally:
        file.close()

    resp_dict = {}
    resp_dict['id_institute']=context_institute.id
    resp_dict['path']=APP_RID_EXPORT_DIR+'\GTO_Result_'+str(context_institute.id)+'.txt'
    return render_to_response('gto_list.html', resp_dict, context_instance=request_context)

def educ_proc_class_transfer(request):
    request_context = RequestContext(request)
    context_institute = request_context['context_institute']
    year = request_context['edu_year']
    resp_dict = {}
    resp_dict['from_year'] = from_year = year-1
    if request.method == 'POST':
        post = request.POST.copy()

        for key in post:
            string = post.getlist(key)
            if len(string) > 1 and string[0]:
                group = ASIOU_G_Education.objects.get(id=string[0])
                group.name = string[1]
                group.level = group.level+1
                group.pk = None
                group.id = None
                group.education_year = year
                group.local_id = get_free_id(ASIOU_G_Education)
                if group.level == 9 or group.level == 11 or group.level == 12:
                    group.final = True
                else:
                    group.final = False

                try:
                    group.save()
                    for group_param in group.type.groups.all():
                        for param in group_param.params.all():
                            v_params = ASIOU_P_ParamValues.objects.filter(institute=context_institute.id, param=param, group_id=group_param.id, record_id=string[0])
                            for v_param in v_params:
                                v_param.pk = None
                                v_param.record_id = group.pk
                                if param.label == 'ge_org_stage' and (group.level == 5 or group.level == 10):
                                    v_param.value_int += 1

                                v_param.save()
                    messages.info(request, u'Группа с именем %s перенесен в учебный год %s-%s ' % (group.name, year, year+1))
                except:
                    messages.error(request, u'Группа с именем %s уже есть в учебном году %s-%s ' % (group.name, year, year+1))

    groups = ASIOU_G_Education.objects.filter(institute=context_institute, education_year=from_year, final=False)
    for g in groups:
        for group_param in g.type.groups.all():
            for param in group_param.params.all():
                if param.label == 'ge_org_stage':
                    if not param.multi:
                        g.stage = ASIOU_P_ParamValues.objects.filter(institute=context_institute.id, param=param, group_id=group_param.id, record_id=g.id)
                        if g.stage:
                            g.stage = g.stage[0].display_value()
                        else:
                            g.stage = ''

    resp_dict['groups'] = groups
    return render_to_response('transfer/class.html', resp_dict, context_instance=request_context)
