# -*- coding: utf-8 -*-
import os.path
from ConfigParser import RawConfigParser

# Django settings for ASIOU project.
DEBUG = False
TEMPLATE_DEBUG = DEBUG

# Base path for project
BASE_PATH = os.path.dirname(__file__).replace('\\','/')

# Dtabase connect settings
dbconf = RawConfigParser()
dbconf.read(BASE_PATH + '/db.ini')

# Application settings
app_conf = RawConfigParser()
app_ini = BASE_PATH + '/options.ini'
if os.path.exists(app_ini) and os.path.isfile(app_ini):
    app_conf.read(app_ini)

DATABASES = {
    'default': {
        'ENGINE':   dbconf.get('database', 'DATABASE_ENGINE'),
        'HOST':     dbconf.get('database', 'DATABASE_HOST'),
        'NAME':     dbconf.get('database', 'DATABASE_NAME'),
        'USER':     dbconf.get('database', 'DATABASE_USER'),
        'PASSWORD': dbconf.get('database', 'DATABASE_PASSWORD'),
        'PORT':     dbconf.get('database', 'DATABASE_PORT')
    }
}

# User profile module
AUTH_PROFILE_MODULE = 'ASIOU_U_UserProfile'

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# On Unix systems, a value of None will cause Django to use the same
# timezone as the operating system.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = 'Europe/Moscow'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'ru-ru'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale
USE_L10N = True

# Absolute path to the directory that holds media.
# Example: "/home/media/media.lawrence.com/"
MEDIA_ROOT = os.path.abspath(BASE_PATH + '/../static')

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash if there is a path component (optional in other cases).
# Examples: "http://media.lawrence.com", "http://example.com/media/"
MEDIA_URL = '/static/'

# URL prefix for admin media -- CSS, JavaScript and images. Make sure to use a
# trailing slash.
# Examples: "http://foo.com/media/", "/media/".
ADMIN_MEDIA_PREFIX = '/media/'

# Make this unique, and don't share it with anybody.
SECRET_KEY = '6c5#1j#0@q02tld2+ktzw9lwo)y2wkg1g_-q9$io57swi+84x$'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)

# Context processors
TEMPLATE_CONTEXT_PROCESSORS = (
    'django.contrib.auth.context_processors.auth',
    'django.core.context_processors.debug',
    'django.core.context_processors.i18n',
    'django.core.context_processors.media',
    'django.contrib.messages.context_processors.messages',
    'django.core.context_processors.request',
    'asiou.context_processors.project_settings',
    'asiou.context_processors.context_institute',
    'asiou.context_processors.context_user_settings',
    'asiou.context_processors.general_menu',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.transaction.TransactionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'asiou.common.middleware.ThreadLocals',
)

# Filename whit urls routing
ROOT_URLCONF = 'urls'

# Folders for search templates
TEMPLATE_DIRS = (
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
    os.path.abspath(BASE_PATH + '/../tpls'),
)

# List of installed apps
INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    # Uncomment the next line to enable the admin:
    'django.contrib.admin',
    # Uncomment the next line to enable admin documentation:
    'django.contrib.admindocs',
    # Base, for templatetags
    'asiou',
    # Accounts
    'asiou.accounts',
    # Edu org module
    'asiou.institute',
    # Ed group
    'asiou.group',
    # Adds properties
    'asiou.properties',
    # Person module
    'asiou.person',
    # Document module
    'asiou.document',
    # User settings module
    'asiou.user_settings',
    # Dict settings module
    'asiou.dict',
    # Reports settings module
    'asiou.reports',
    # Education process
    'asiou.educ_proc',
    # Dou queue
    'asiou.douq',
    # Generate reports
    #'asiou.g_reports',
)

# SMTP PARAMS
EMAIL_HOST = 'smtp.yandex.ru'
EMAIL_HOST_USER = 'crazyburn@zavolga.net'
EMAIL_HOST_PASSWORD = 'as'
EMAIL_PORT = 28

# Login params
LOGIN_URL = '/accounts/login/'
LOGIN_REDIRECT_URL = '/'
LOGOUT_URL = '/accounts/logout/'
PROFILE_URL = '/accounts/profile/'

# Муниципальные районы
APP_MR_CHOICES = (
    (11, 'Большесельский район'),
    (12, 'Борисоглебский район'),
    (13, 'Брейтовский район'),
    (14, 'Гаврилов-Ямский район'),
    (15, 'Даниловский район'),
    (16, 'Любимский район'),
    (17, 'Мышкинский район'),
    (18, 'Некоузский район'),
    (19, 'Некрасовский район'),
    (20, 'Первомайский район'),
    (22, 'Переславский район'),
    (23, 'Пошехонский район'),
    (24, 'Ростовский район'),
    (29, 'Рыбинский район'),
    (26, 'Тутаевский район'),
    (27, 'Угличский район'),
    (28, 'Ярославский район'),
    (21, 'г. Переславль-Залесский'),
    (25, 'г. Рыбинск'),
    (8,  'г. Ярославль'),
    (0,  'ДО ЯО'),
)

# Dict params
APP_KLADR_UPDATE_URL = 'http://127.0.0.1:8001/dict/kladr_update/'

# Kladr load params
APP_KLADR_LOAD_INC = 1024

# Kladr timeout update
APP_KLADR_UPDATE_PROCESS_TIMEOUT = 5000

# Kladr blocs for downloading file
APP_KLADR_DL_BLOCK = 8192

# Kladr download url
APP_KLADR_DL_URL = 'http://www.gnivc.ru/html/gnivcsoft/KLADR/BASE.7z'

# Kladr default filename
APP_KLADR_DEF_FNAME = 'base.7z'

# Dict params
APP_KLADR_UPDATE_PROCESS_URL = '/dict/kladr_update_process/'
APP_KLADR_PROCESS_FILE_NAME = 'process.log'

# Kladr count attamps to load status file
APP_KLADR_ATTAMPS_LOAD = 10

# Specific application settings
APP_NAME = 'АСИОУ'
APP_VERSION = '7.1'
APP_COPYRIGHT = '© 2012 Департамент образования Ярославской области'
APP_INSTITUTE_URL = '/institute/school/edit/'
APP_LOGO = 'logo.gif'
APP_FAKE_USER_ID = 1
APP_OKSM_RU_CODE = 643

# Текущая верия экспорта в дневник
APP_RID_EXPORT_VER = '3.1'

# Код региона по умолчанию
APP_REGION_CODE = '76000000000'

# Константы с типом прав
APP_RIGHT_READ  = 'r'
APP_RIGHT_WRITE = 'w'

# Каталог с файлами данных из АСИОУ 6.0 for Windows
APP_CONV_DIR = os.path.abspath(BASE_PATH + '/../conv')

# Каталог с SQL файлами для первичной инициализации БД
APP_INITDB_DIR = os.path.abspath(BASE_PATH + '/../sql/initdb')

# Каталог с SQL файлами для обновления структуры БД
APP_UPDATEDB_DIR = os.path.abspath(BASE_PATH + '/../sql/updatedb')

# Файл содержащий текущую версию структуры БД
APP_DBVER_FILE = os.path.abspath(BASE_PATH + '/db.version')

# Переопределим из конфига
if app_conf.has_option('application', 'NAME'):
    APP_NAME = app_conf.get('application', 'NAME')

if app_conf.has_option('application', 'VERSION'):
    APP_VERSION = app_conf.get('application', 'VERSION')

if app_conf.has_option('application', 'COPYRIGHT'):
    APP_COPYRIGHT = app_conf.get('application', 'COPYRIGHT')

if app_conf.has_option('application', 'INSTITUTE_URL'):
    APP_INSTITUTE_URL = app_conf.get('application', 'INSTITUTE_URL')

if app_conf.has_option('application', 'LOGO'):
    APP_LOGO = app_conf.get('application', 'LOGO')

if app_conf.has_option('application', 'REGION_CODE'):
    APP_REGION_CODE = app_conf.get('application', 'REGION_CODE')

# Organization type
APP_ORG_TYPE = 'school'
if app_conf.has_option('application', 'ORG_TYPE'):
    APP_ORG_TYPE = app_conf.get('application', 'ORG_TYPE')

# Cookies
APP_COOKIES_PREFIX = 'asiou_'
APP_COOKIES_MAX_AGE = 86400 * 365 * 10

# Paging
APP_DEFAULT_PERPAGE = 20
APP_MAX_PERPAGE_ITEMS = 2000
APP_ITEMS_PERPAGE = (10, 20, 30, 50, 100, )

# Student type prefix
APP_STUDENT_PREFIX = 'student'
APP_WORKER_PREFIX = 'teach'

# Temporary directory
APP_TEMPDIR = os.path.abspath(BASE_PATH + '/../temp')

# Reports
APP_REP_LABEL_PREF = 'l_'

# Flags params
APP_FLAG_PREFIX = 'flag_'
APP_FLAG_DIR = os.path.abspath(BASE_PATH + '/../temp')

# RID import params
APP_RID_EXPORT_DIR = os.path.abspath(BASE_PATH + '/../temp')

# System roles
ROLE_ALL               = 'all'
ROLE_OOU_ADMIN         = 'oou_admin'
ROLE_OOU_REGION_ADMIN  = 'oou_region_admin'
ROLE_OOU_MR_ADMIN      = 'oou_mr_admin'
ROLE_OOU_SCHOOL_ADMIN  = 'oou_school_admin'
ROLE_OOU_TEACHER       = 'oou_teacher'
#ROLE_OOU_OPERATOR_GTO = 'oou_operator_gto'

OOU_USER_ROLES_CHOICES = (
    (ROLE_OOU_ADMIN, 'Суперпользователь'),
    (ROLE_OOU_REGION_ADMIN, 'Региональный оператор'),
    (ROLE_OOU_MR_ADMIN, 'Муниципальный оператор'),
    (ROLE_OOU_SCHOOL_ADMIN, 'Администратор ОУ'),
    (ROLE_OOU_TEACHER, 'Учитель-предметник'),
#    (ROLE_OOU_OPERATOR_GTO, 'Оператор ГТО'),
)

UDOD_USER_ROLES_CHOICES = (
    (ROLE_OOU_ADMIN, 'Суперпользователь'),
    (ROLE_OOU_REGION_ADMIN, 'Региональный оператор'),
    (ROLE_OOU_MR_ADMIN, 'Муниципальный оператор'),
    (ROLE_OOU_SCHOOL_ADMIN, 'Администратор ОУ'),
    (ROLE_OOU_TEACHER, 'Педагог'),
)

PROF_USER_ROLES_CHOICES = (
    (ROLE_OOU_ADMIN, 'Суперпользователь'),
    (ROLE_OOU_REGION_ADMIN, 'Региональный оператор'),
    (ROLE_OOU_MR_ADMIN, 'Муниципальный оператор'),
    (ROLE_OOU_SCHOOL_ADMIN, 'Администратор ОУ'),
    (ROLE_OOU_TEACHER, 'Преподаватель'),
)

ROLE_DOU_ADMIN         = 'dou_admin'
ROLE_DOU_REGION_ADMIN  = 'dou_region_admin'
ROLE_DOU_MR_ADMIN      = 'dou_mr_admin'
ROLE_DOU_MANAGER       = 'dou_org_manager'
ROLE_DOU_QUEUE_MANAGER = 'dou_queue_manager'
ROLE_DOU_COMPL_MANAGER = 'dou_compl_manager'
ROLE_DOU_SUPPORT       = 'dou_support'

DOU_USER_ROLES_CHOICES = (
    (ROLE_DOU_ADMIN, 'Администратор'),
    (ROLE_DOU_REGION_ADMIN, 'Региональный оператор'),
    (ROLE_DOU_MR_ADMIN, 'Муниципальный оператор'),
    (ROLE_DOU_MANAGER, 'Руководитель ДОУ'),
    (ROLE_DOU_QUEUE_MANAGER, 'Специалист по приему заявлений'),
    (ROLE_DOU_COMPL_MANAGER, 'Специалист по комплектованию'),
    (ROLE_DOU_SUPPORT, 'Служба поддержки'),
)

# Role choices by org type
ROLE_CHOICES_BY_ORG_TYPE = {
    'school': OOU_USER_ROLES_CHOICES,
    'dou': DOU_USER_ROLES_CHOICES,
    'udod': UDOD_USER_ROLES_CHOICES,
    'prof': PROF_USER_ROLES_CHOICES
}

# Default role by org type
ROLE_DEFAULT_BY_ORG_TYPE = {
    'school': ROLE_OOU_TEACHER,
    'dou': ROLE_DOU_QUEUE_MANAGER,
    'udod': ROLE_OOU_TEACHER,
    'prof': ROLE_OOU_TEACHER
}

# Menu choices by org type
MENU_BY_ORG_TYPE = {
    'school': 'menu.xml',
    'dou': 'dou_menu.xml',
    'udod': 'menu_udod.xml',
    'prof': 'menu_prof.xml'
}

LOGIN_REGEXP_BY_ORG_TYPE = {
    'school': '^[0-9]+$',
    'dou': '^[a-zA-Z0-9_@\+\-\.]+$',
    'udod': '^[0-9]+$',
    'prof': '^[0-9]+$'
}

# Метка для объекта типа "Статус в очереди"
APP_DOUQ_STATUS_LABEL = 'douq_status'
APP_DOUQ_HELPER_START_ID = 990000

# Сколько дней давать на то чтобы внести данные при отсутствии интернета
APP_DOUQ_BACKEDIT_DAYS = 7

# Возвраст (лет) в котором ребенок должен уходить из очереди
APP_DOUQ_DEFACE_YEARS_OLD = 7

# Делать ли обезличивание при периодических обработках и через сколько дней после статуса "Архив"
APP_DOUQ_DO_DEFACE = True
APP_DOUQ_DEFACE_DAYS = 7

# Current usage role choices
APP_CURRENT_ROLE_CHOICES = ROLE_CHOICES_BY_ORG_TYPE[APP_ORG_TYPE]
APP_DEFAULT_ROLE = ROLE_DEFAULT_BY_ORG_TYPE[APP_ORG_TYPE]
APP_MENU_FILE = MEDIA_ROOT + '/' + MENU_BY_ORG_TYPE[APP_ORG_TYPE]
APP_LOGIN_REGEXP = LOGIN_REGEXP_BY_ORG_TYPE[APP_ORG_TYPE]

# Границы учебного года по умолчанию
APP_EDU_YEAR_BORDER_DAY = 1
APP_EDU_YEAR_BORDER_MONTH = 9

# Базовый URL для управления группами
APP_GROUP_URL = '/group/'
APP_EDUC_PROC_URL = '/educ_proc/'
APP_EP_GROUP_URL = '/educ_proc/ep_group/'
APP_EP_MARKS_URL = '/educ_proc/ep_marks/'
APP_PERSON_BASE_URL = '/person/'
APP_PERSON_PUPIL_URL = '/person/pup/'

# Через скорлько строк повторять заголовок с датами
MARK_DATE_HEAD_ROWS = 15

# Подстроки в названиях типов приказа
APP_DOCUM_IN = '_in'
APP_DOCUM_DEL = '_out'
APP_PERSON_PREFIX = 'work'
APP_DOCUM_ROOT = '/document/'
APP_NEW_DOCUM_URL = APP_DOCUM_ROOT + 'select_otype/'

# Адреса для редиректов при создании новой заявки
APP_DOUQ_SELECT_URL = '/douq/dou_queue/related/%s/ASIOU_DouqSelect/'
APP_DOUQ_STATUS_URL = '/douq/dou_queue/related/%s/ASIOU_DouqStatus/'
APP_DOUQ_URL = '/douq/dou_queue/'
APP_DOUQ_SUCCESS_URL = '/douq/dou_queue/success/%s/'

# Максимальный возраст ребенка для ДОУ
APP_DOUQ_MAX_CHILD_OLD = 7

# Максимальное время у группы при которой она считается временной
APP_DOUQ_TEMP_MAX_HOUR = 5

# ID служебного предмета "Классный час"
CLASS_HOUR_ID = 273

# ID служебного предмета ГТО
GTO_ID = 272

# Коды отметок
APP_MARK_ABSENT = -1 # Отсутствовал
APP_MARK_ILL = -2    # Болел
APP_MARK_LATE = -3   # Опоздал
APP_MARK_ADD = -10   # Плюс
APP_MARK_MINUS = -11 # Минус
APP_MARK_DONE = -20  # Выполнено
APP_MARK_NDONE = -21 # Не выполнено
APP_MARK_MAST = -22  # Освоил
APP_MARK_NMAST = -23 # Не освоил
APP_MARK_PASS = -24  # Зачтено
APP_MARK_NPASS = -25 # Не зачтено
APP_MARK_YES = -26   # Да
APP_MARK_NO = -27    # Нет
APP_MARK_BAD = -30   # Плохо
APP_MARK_SATIS = -31 # Удовлетворительно
APP_MARK_GOOD = -32  # Хорошо
APP_MARK_FINE = -33  # Отлично

# Кол-во столбцов выбора при распечатке журнала
APP_PRINT_COUNT_MARKS = 15
